////////////////////////////////////////////////////////////////////////////////
//  
//  AbstractDirectedGraph.m
//  
//  KPMultigraph
//  
//  Created by Konstantin Pavlikhin on 05/09/11.
//  
////////////////////////////////////////////////////////////////////////////////

#import "AbstractDirectedGraph.h"

@implementation AbstractDirectedGraph

#pragma mark - Graph protocol overrides

- (NSUInteger) degreeOfVertex: (vertexType) vertex
{
  // THINK: What about self-loops?
  return [self indegreeOfVertex: vertex] + [self outdegreeOfVertex: vertex];
}

- (NSEnumerator*) possibleWaysOutEnumeratorOf: (vertexType) vertex
{
  return [[self outgoingEdgesSetOf: vertex] objectEnumerator];
}

- (vertexType) oppositeVertexOfEdge: (edgeType) edge currentVertex: (vertexType) vertex
{
  vertexType tailVertex = [self tailVertexOfEdge: edge];
  
  vertexType headVertex = [self headVertexOfEdge: edge];
  
  if([vertex isEqual: tailVertex])
  {
    return headVertex;
  }
  else if([vertex isEqual: headVertex])
  {
    return tailVertex;
  }
  
  // TODO: raise exception "Invalid Argument".
  
  return nil;
}

- (NSSet*) openNeighbourhoodSetOf: (vertexType) vertex
{
  return [[self directPredcessorsSetOfVertex: vertex] setByAddingObjectsFromSet: [self directSuccessorsSetOfVertex: vertex]];
}

#pragma mark - Operations with vertices

- (NSSet*) directSuccessorsSetOfVertex: (vertexType) vertex
{
  [self subclassResponsibility: _cmd];
  
  return nil;
}

- (NSUInteger) directSuccessorsCountOfVertex: (vertexType) vertex
{
  return [[self directSuccessorsSetOfVertex: vertex] count];
}

- (BOOL) isVertex: (vertexType) vertex1 directSuccessorOf: (vertexType) vertex2
{
  return [[self directSuccessorsSetOfVertex: vertex2] containsObject: vertex1];
}

- (NSSet*) directPredcessorsSetOfVertex: (vertexType) vertex
{
  [self subclassResponsibility: _cmd];
  
  return nil;
}

- (NSUInteger) directPredcessorsCountOfVertex: (vertexType) vertex
{
  return [[self directPredcessorsSetOfVertex: vertex] count];
}

- (BOOL) isVertex: (vertexType) vertex1 directPredcessorOf: (vertexType) vertex2
{
  return [[self directPredcessorsSetOfVertex: vertex2] containsObject: vertex1];
}

////////////////////////////////////////////////////////////////////////////////

- (NSUInteger) indegreeOfVertex: (vertexType) vertex
{
  return [[self incomingEdgesSetOf: vertex] count];
}

- (NSUInteger) outdegreeOfVertex: (vertexType) vertex
{
  return [[self outgoingEdgesSetOf: vertex] count];
}

- (BOOL) isLocalSourceVertex: (vertexType) vertex
{
  return [self indegreeOfVertex: vertex] == 0;
}

- (BOOL) isGlobalSourceVertex: (vertexType) vertex
{
  [self subclassResponsibility: _cmd];
  
  return NO;
}

- (BOOL) isLocalSinkVertex: (vertexType) vertex
{
  return [self outdegreeOfVertex: vertex] == 0;
}

- (BOOL) isGlobalSinkVertex: (vertexType) vertex
{
  [self subclassResponsibility: _cmd];
  
  return NO;
}

#pragma mark - Operations with vertices & edges

- (edgeType) edgeFrom: (vertexType) u to: (vertexType) v
{
  [self subclassResponsibility: _cmd];
  
  return nil;
}

- (NSSet*) allEdgesFrom: (vertexType) u to: (vertexType) v
{
  [self subclassResponsibility: _cmd];
  
  return nil;
}

- (BOOL) containsEdgeFrom: (vertexType) u to: (vertexType) v
{
  return [self edgeFrom: u to: v] != nil;
}

- (NSSet*) incomingEdgesSetOf: (vertexType) vertex
{
  [self subclassResponsibility: _cmd];
  
  return nil;
}

- (NSSet*) outgoingEdgesSetOf: (vertexType) vertex
{
  [self subclassResponsibility: _cmd];
  
  return nil;
}

- (vertexType) tailVertexOfEdge: (edgeType) edge
{
  [self subclassResponsibility: _cmd];
  
  return nil;
}

- (vertexType) headVertexOfEdge: (edgeType) edge
{
  [self subclassResponsibility: _cmd];
  
  return nil;
}

- (BOOL) isVertex: (vertexType) vertex tailOfEdge: (edgeType) edge
{
  return [[self tailVertexOfEdge: edge] isEqual: vertex];
}

- (BOOL) isVertex: (vertexType) vertex headOfEdge: (edgeType) edge
{
  return [[self headVertexOfEdge: edge] isEqual: vertex];
}

@end
