////////////////////////////////////////////////////////////////////////////////
//  
//  DirectedGraph.h
//  
//  KPMultigraph
//  
//  Created by Konstantin Pavlikhin on 30/08/11.
//  
////////////////////////////////////////////////////////////////////////////////

#import "Graph.h"

@protocol DirectedGraph <Graph>

#pragma mark - Operations with vertices

// Отходим от понятий "u is adjacent to/from v" и заменяем их более простыми словами predcessors/successors.

- (NSSet*) directSuccessorsSetOfVertex: (vertexType) vertex;

- (NSUInteger) directSuccessorsCountOfVertex: (vertexType) vertex;

- (BOOL) isVertex: (vertexType) vertex1 directSuccessorOf: (vertexType) vertex2;


- (NSSet*) directPredcessorsSetOfVertex: (vertexType) vertex;

- (NSUInteger) directPredcessorsCountOfVertex: (vertexType) vertex;

- (BOOL) isVertex: (vertexType) vertex1 directPredcessorOf: (vertexType) vertex2;

////////////////////////////////////////////////////////////////////////////////

/*!
 * Returns indegree of a vertex.
 *
 * @param vertex Vertex for which to compute an indegree.
 *
 * @return Vertex indegree value. Should return NSNotFound if vertex doesn't belong to the graph.
 */
- (NSUInteger) indegreeOfVertex: (vertexType) vertex;

/*!
 * Returns outdegree of a vertex.
 *
 * @param vertex Vertex for which to compute an outdegree.
 *
 * @return Vertex outdegree value. Should return NSNotFound if vertex doesn't belong to the graph.
 */
- (NSUInteger) outdegreeOfVertex: (vertexType) vertex;

// * A local source is a node of a directed graph with no entering edges.
- (BOOL) isLocalSourceVertex: (vertexType) vertex;

// * A global source is a node of a directed graph which reaches all other nodes.
- (BOOL) isGlobalSourceVertex: (vertexType) vertex;

// * A local sink is a node of a directed graph with no exiting edges, also called a terminal.
- (BOOL) isLocalSinkVertex: (vertexType) vertex;

// * A global sink is a node of a directed graph which is reached by all directed edges.
- (BOOL) isGlobalSinkVertex: (vertexType) vertex;

#pragma mark - Operations with vertices & edges

/*!
 * Returns a directed edge going from u to v.
 *
 * @return An edge connecting given vertices, if these vertices exist in graph. nil otherwise.
 */
- (edgeType) edgeFrom: (vertexType) u to: (vertexType) v;

/*!
 * Returns a set of directed edges going from u to v.
 *
 * @return A set of edges that connect given vertices, if these vertices exist in graph. nil otherwise.
 */
- (NSSet*) allEdgesFrom: (vertexType) u to: (vertexType) v;

// * Returns YES if graph contains an edge between given vertices u & v.
- (BOOL) containsEdgeFrom: (vertexType) u to: (vertexType) v;

// Отходим от понятий "e is incident to/from u" и заменяем их более простыми словами incoming/outcoming.

- (NSSet*) incomingEdgesSetOf: (vertexType) vertex;

- (NSSet*) outgoingEdgesSetOf: (vertexType) vertex;

// Отходим от понятий "u is incident to/from e" и заменяем их более простыми словами tail/head.

/*!
 * Returns the tail vertex of an edge.
 *
 * @return Tail vertex of an edge or nil if edge is nil or doesn't belong to the graph.
 */
- (vertexType) tailVertexOfEdge: (edgeType) edge;

/*!
 * Returns the head vertex of an edge.
 *
 * @return Head vertex of an edge or nil if edge is nil or doesn't belong to the graph.
 */
- (vertexType) headVertexOfEdge: (edgeType) edge;

- (BOOL) isVertex: (vertexType) vertex tailOfEdge: (edgeType) edge;

- (BOOL) isVertex: (vertexType) vertex headOfEdge: (edgeType) edge;

@end

#pragma mark - Mutable Version

@protocol MutableDirectedGraph <MutableGraph, DirectedGraph>

- (void) addEdge: (edgeType) edge from: (vertexType) u to: (vertexType) v;

- (edgeType) removeEdgeFrom: (vertexType) u to: (vertexType) v;

@end
