////////////////////////////////////////////////////////////////////////////////
//  
//  DirectedSparsePseudograph.m
//  
//  KPMultigraph
//  
//  Created by Konstantin Pavlikhin on 02/09/11.
//  
////////////////////////////////////////////////////////////////////////////////

#import "DirectedSparsePseudograph.h"

NSString* const VertexToPairOfEdgeSets = @"vertexToPairOfEdgeSets";

NSString* const EdgeToPairOfVertices = @"edgeToPairOfVertices";

@implementation DirectedSparsePseudograph

@synthesize vertexToPairOfEdgeSets;

@synthesize edgeToPairOfVertices;

+ (NSMapTable*) nonCopyingMapTable
{
  NSPointerFunctionsOptions keyOptions = NSPointerFunctionsStrongMemory | NSPointerFunctionsObjectPointerPersonality;
  
  NSPointerFunctionsOptions valueOptions = NSPointerFunctionsStrongMemory | NSPointerFunctionsObjectPointerPersonality;
  
  return [NSMapTable mapTableWithKeyOptions: keyOptions valueOptions: valueOptions];
}

- (id) init
{
  self = [super init];
  
  if(!self) return nil;
  
  vertexToPairOfEdgeSets = [[[self class] nonCopyingMapTable] retain];
  
  edgeToPairOfVertices = [[[self class] nonCopyingMapTable] retain];
  
  return self;
}

- (void) dealloc
{
  [vertexToPairOfEdgeSets release], vertexToPairOfEdgeSets = nil;
  
  [edgeToPairOfVertices release], edgeToPairOfVertices = nil;
  
  [super dealloc];
}

- (NSSet*) allEdgesFrom: (vertexType) u to: (vertexType) v
{
  NSSet* outgoingEdges = [[vertexToPairOfEdgeSets objectForKey: u] object2];
  
  BOOL (^predicate)(id, BOOL*) = ^(id object, BOOL* stop)
  {
    return [[self headVertexOfEdge: object] isEqual: v];
  };
  
  return [outgoingEdges objectsPassingTest: predicate];
}

#pragma mark - Graph protocol implementation

- (NSSet*) vertexSet
{
  // HACK: Метод allKeys объявлен в private-категории NSMapTable :(
  return [NSSet setWithArray: [(id)vertexToPairOfEdgeSets allKeys]];
}

- (BOOL) containsVertex: (vertexType) vertex
{
  return [vertexToPairOfEdgeSets objectForKey: vertex] != nil;
}

- (NSUInteger) vertexCount
{
  return [vertexToPairOfEdgeSets count];
}

- (NSSet*) edgeSet
{
  // HACK: Метод allKeys объявлен в private-категории NSMapTable :(
  return [NSSet setWithArray: [(id)edgeToPairOfVertices allKeys]];
}

- (BOOL) isLoopEdge: (edgeType) edge
{
  vertexType tailVertex = [self tailVertexOfEdge: edge];
  
  return [self isVertex: tailVertex headOfEdge: edge];
}

#pragma mark - DirectedGraph protocol implementation

// TODO: make template method from these two below:

- (NSSet*) directSuccessorsSetOfVertex: (vertexType) vertex
{
  NSSet* outgoingEdges = [[vertexToPairOfEdgeSets objectForKey: vertex] object2];
  
  NSEnumerator* edgeEnumerator = [outgoingEdges objectEnumerator];
  
  edgeType edge = nil;
  
  NSMutableSet* successors = [NSMutableSet set];
  
  while(edge = [edgeEnumerator nextObject]) [successors addObject: [self headVertexOfEdge: edge]];
  
  return [[successors copy] autorelease];
}

- (NSSet*) directPredcessorsSetOfVertex: (vertexType) vertex
{
  NSSet* incomingEdges = [[vertexToPairOfEdgeSets objectForKey: vertex] object1];
  
  NSEnumerator* edgeEnumerator = [incomingEdges objectEnumerator];
  
  edgeType edge = nil;
  
  NSMutableSet* predcessors = [NSMutableSet set];
  
  while(edge = [edgeEnumerator nextObject]) [predcessors addObject: [self tailVertexOfEdge: edge]];
  
  return [[predcessors copy] autorelease];
}

- (BOOL) isGlobalSourceVertex: (vertexType) vertex
{
  // TODO: implement me.
  
  return NO;
}

- (BOOL) isGlobalSinkVertex: (vertexType) vertex
{
  // TODO: implement me.
  
  return NO;
}

- (edgeType) edgeFrom: (vertexType) u to: (vertexType) v
{
  NSSet* outgoingEdges = [[vertexToPairOfEdgeSets objectForKey: u] object2];
  
  NSEnumerator* edgeEnumerator = [outgoingEdges objectEnumerator];
  
  edgeType edge = nil;
  
  while(edge = [edgeEnumerator nextObject])
  {
    if([[self headVertexOfEdge: edge] isEqual: v]) return edge;
  }
  
  return nil;
}

- (NSSet*) incomingEdgesSetOf: (vertexType) vertex
{
  return [[[[vertexToPairOfEdgeSets objectForKey: vertex] object1] copy] autorelease];
}

- (NSSet*) outgoingEdgesSetOf: (vertexType) vertex
{
  return [[[[vertexToPairOfEdgeSets objectForKey: vertex] object2] copy] autorelease];
}

- (vertexType) tailVertexOfEdge: (edgeType) edge
{
  return [[edgeToPairOfVertices objectForKey: edge] object1];
}

- (vertexType) headVertexOfEdge: (edgeType) edge
{
  return [[edgeToPairOfVertices objectForKey: edge] object2];
}

- (void) copyFieldsFrom: (DirectedSparsePseudograph*) source to: (DirectedSparsePseudograph*) destination
{
  [destination setVertexToPairOfEdgeSets: [source vertexToPairOfEdgeSets]];
  
  [destination setEdgeToPairOfVertices: [source edgeToPairOfVertices]];
}

#pragma mark - NSCopying protocol implementation

// Immutable-копия immutable-объекта.
- (id) copyWithZone: (NSZone*) zone
{
  // Терять нечего — никто из объектов не может изменить свое содержимое.
  return [self retain];
}

#pragma mark - NSMutableCopying implementation

// Mutable-копия immutable-объекта.
- (id) mutableCopyWithZone: (NSZone*) zone
{
  // Создаем экземпляр мутабельного сабкласса.
  MutableDirectedSparsePseudograph* mutableCopy = [[MutableDirectedSparsePseudograph allocWithZone: zone] init];
  
  // Заполняем поля копиями, чтобы создаваемый объект не мог повлиять на состояние текущего.
  [self copyFieldsFrom: self to: mutableCopy];
  
  return mutableCopy;
}

#pragma mark - NSCoding protocol implementation

// TODO: switch to AutoCoder?
- (void) encodeWithCoder: (NSCoder*) coder
{
  [coder encodeObject: vertexToPairOfEdgeSets forKey: VertexToPairOfEdgeSets];
  
  [coder encodeObject: edgeToPairOfVertices forKey: EdgeToPairOfVertices];
}

- (id) initWithCoder: (NSCoder*) decoder
{
  self = [self init];
  
  if(!self) return nil;
  
  void (^carryElements)(NSString*) = ^(NSString* name)
  {
    NSMapTable* source = [decoder decodeObjectForKey: name];
    
    NSEnumerator* enumerator = [source keyEnumerator];
    
    id key = nil;
    
    NSMapTable* dest = [self valueForKey: name];
    
    while(key = [enumerator nextObject])
    {
      [dest setObject: [source objectForKey: key] forKey: key];
    }
  };
  
  carryElements(VertexToPairOfEdgeSets);
  
  carryElements(EdgeToPairOfVertices);
  
  return self;
}

#pragma mark - NSFastEnumeration

// for(id object in someGraph) энумерует вершины!
- (NSUInteger) countByEnumeratingWithState: (NSFastEnumerationState*) state objects: (id*) stackbuf count: (NSUInteger) len
{
  return [vertexToPairOfEdgeSets countByEnumeratingWithState: state objects: stackbuf count: len];
}

@end

#pragma mark - Mutable Version

@implementation MutableDirectedSparsePseudograph

#pragma mark - Operations with vertices

- (void) addVertex: (vertexType) vertex
{
  if([self containsVertex: vertex]) return;
  
  [vertexToPairOfEdgeSets setObject: [Pair pairWithObject: [NSMutableSet set] andObject: [NSMutableSet set]] forKey: vertex];
}

- (void) removeVertex: (vertexType) vertex
{
  Pair* edgeSets = [vertexToPairOfEdgeSets objectForKey: vertex];
  
  // Remove incoming.
  [self removeEdgesFromSet: [edgeSets object1]];
  
  // Remove outgoing.
  [self removeEdgesFromSet: [edgeSets object2]];
  
  // Remove vertex.
  [vertexToPairOfEdgeSets removeObjectForKey: vertex];
}

- (void) removeVerticesFromSet: (NSSet*) vertices
{
  for(vertexType vertex in vertices) [self removeVertex: vertex];
}

- (void) removeAllVertices
{
  [edgeToPairOfVertices removeAllObjects];
  
  [vertexToPairOfEdgeSets removeAllObjects];
}

#pragma mark - Operations with edges

- (void) addEdge: (edgeType) edge from: (vertexType) u to: (vertexType) v
{
  // Incoming.
  [[[vertexToPairOfEdgeSets objectForKey: v] object1] addObject: edge];
  
  // Outgoing.
  [[[vertexToPairOfEdgeSets objectForKey: u] object2] addObject: edge];
  
  Pair* tailAndHead = [Pair pairWithObject: u andObject: v];
  
  [edgeToPairOfVertices setObject: tailAndHead forKey: edge];
}

- (void) removeEdge: (edgeType) edge
{
  Pair* tailAndHead = [edgeToPairOfVertices objectForKey: edge];
  
  // Incoming.
  [[[vertexToPairOfEdgeSets objectForKey: [tailAndHead object1]] object2] removeObject: edge];
  
  // Outgoing.
  [[[vertexToPairOfEdgeSets objectForKey: [tailAndHead object2]] object1] removeObject: edge];
  
  [edgeToPairOfVertices removeObjectForKey: edge];
}

- (void) removeEdgesFromSet: (NSSet*) edges
{
  for(edgeType edge in [edges allObjects]) [self removeEdge: edge];
}

- (void) removeAllEdges
{
  // HACK: Метод allKeys объявлен в private-категории NSMapTable :(
  NSArray* allKeys = [[[(id)edgeToPairOfVertices allKeys] copy] autorelease];
  
  for(edgeType key in allKeys) [self removeEdge: key];
}

- (edgeType) removeEdgeFrom: (vertexType) u to: (vertexType) v
{
  edgeType edge = [self edgeFrom: u to: v];
  
  if(edge) [self removeEdge: edge];
  
  return edge;
}

#pragma mark - NSCopying protocol implementation

// Immutable-копия mutable-объекта.
- (id) copyWithZone: (NSZone*) zone
{
  // Создаем экземпляр немутабельного базового класса.
  DirectedSparsePseudograph* immutableCopy = [[DirectedSparsePseudograph allocWithZone: zone] init];
  
  // Заполняем поля копиями, чтобы создаваемый объект не изменялся при модификациях текущего.
  [self copyFieldsFrom: self to: immutableCopy];
  
  return immutableCopy;
}

#pragma mark - NSMutableCopying implementation

// Mutable-копия mutable-объекта.
- (id) mutableCopyWithZone: (NSZone*) zone
{
  // Используем [self class], чтобы сабклассы MutableDirectedSparsePseudograph могли нормально дополнить процесс mutable-копирования.
  id mutableCopy = [[[self class] allocWithZone: zone] init];
  
  // Заполняем поля копиями, чтобы создаваемый объект и текущий не влияли друг на друга.
  [self copyFieldsFrom: self to: mutableCopy];
  
  return mutableCopy;
}

@end
