////////////////////////////////////////////////////////////////////////////////
//  
//  EdgeClassifier.h
//  
//  KPMultigraph
//  
//  Created by Konstantin Pavlikhin on 13/09/11.
//  
////////////////////////////////////////////////////////////////////////////////

#import "EdgeClassifier.h"

#import "EdgeClassifierDelegate.h"

#import "TimeStamper.h"

@implementation AbstractEdgeClassifier

- (id) initWithDelegate: (NSObject<EdgeClassifierDelegate>*) d
{
  self = [super init];
  
  if(!self) return nil;
  
  delegate = d;
  
  treeEdges = [[NSHashTable hashTableWithOptions: NSHashTableStrongMemory | NSHashTableObjectPointerPersonality] retain];
  
  return self;
}

- (void) dealloc
{
  [treeEdges release], treeEdges = nil;
  
  [super dealloc];
}

#pragma mark - TraversalVisitor

- (void) connectedVerticesTraversalDidStart
{
  [treeEdges removeAllObjects];
}

- (void) willTraverseEdge: (edgeType) e
{
  currentEdge = e;
}

// Tree Edges (corresponds both to the BFT & DFT).
- (void) foundUndiscoveredVertex: (vertexType) v
{
  // foundUndiscoveredVertex вызывается с параметром v == стартовой вершине, когда willTraverseEdge еще не вызывался.
  if(currentEdge)
  {
    [[delegate ifResponds] edgeClassifiedAsTreeEdge: currentEdge];
    
    [treeEdges addObject: currentEdge];
  }
}

@end

#pragma mark - BFTEdgeClassifier

@implementation BFTEdgeClassifier

- (void) foundDiscoveredVertex: (vertexType) v
{
  [[delegate ifResponds] edgeClassifiedAsCrossEdge: currentEdge];
}

- (void) foundProcessedVertex: (vertexType) v
{
  if([treeEdges containsObject: currentEdge]) return;
  
  [[delegate ifResponds] edgeClassifiedAsBackEdge: currentEdge];
}

@end

#pragma mark - DFTEdgeClassifier

@implementation DFTEdgeClassifier

@synthesize timeStamper;

- (id) initWithTimeStamper: (TimeStamper*) ts delegate: (NSObject<EdgeClassifierDelegate>*) d
{
  self = [super initWithDelegate: d];
  
  if(!self) return nil;
  
  [self setTimeStamper: ts];
  
  return self;
}

- (void) dealloc
{
  [timeStamper release], timeStamper = nil;
  
  [super dealloc];
}

- (void) willVisitVertex: (vertexType) v
{
  currentVertex = v;
}

- (void) foundDiscoveredVertex: (vertexType) v
{
  if([treeEdges containsObject: currentEdge]) return;
  
  [[delegate ifResponds] edgeClassifiedAsBackEdge: currentEdge];
}

- (void) foundProcessedVertex: (vertexType) v
{
  if([timeStamper previsitTimeOfVertex: currentEdge] <= [timeStamper previsitTimeOfVertex: v])
  {
    [[delegate ifResponds] edgeClassifiedAsForwardEdge: currentEdge];
    
    return;
  }
  
  if([timeStamper previsitTimeOfVertex: currentEdge] > [timeStamper previsitTimeOfVertex: v])
  {
    [[delegate ifResponds] edgeClassifiedAsCrossEdge: currentEdge];
    
    return;
  }
  
  [NSException raise: NSInternalInconsistencyException format: @"Could not classify edge."];
}

@end
