////////////////////////////////////////////////////////////////////////////////
//  
//  EdgeClassifierDelegate.h
//  
//  KPMultigraph
//  
//  Created by Konstantin Pavlikhin on 13/09/11.
//  
////////////////////////////////////////////////////////////////////////////////

#import <Foundation/Foundation.h>

#import "ElementTypes.h"

/*!
 * Distance
 *
 * BFS Edge Classification:
 * — Tree Edges
 *   For each tree edge (u, v), we have d[v] = d[u] + 1.
 *
 * — Back Edges
 *   For each back edge (u, v), we have 0 ≤ d[v] ≤ d[u].
 *
 * — Cross Edges
 *   For each cross edge (u, v), we have d[v] ≤ d[u] + 1.
 *
 *   There are no forward edges, because there are no edges (u,v) with d(v) > d(u) + 1.
 */

/*!
 * Discovery and Process Times.
 *
 * DFS Edge Classification:
 * — Tree Edges
 *   *GRAY* —> *WHITE*
 *   This indicates that d[u] < d[v], (u’s discovery time is less than v’s discovery time), so a tree edge points from a “low” to a “high” node.
 *
 * — Back Edges
 *   *GRAY* —> *GRAY*
 *   pre[v] > pre[w], and post[w] is undefined
 *   back edge if and only if d[v] < d[u] < f[u] < f[v]
 *   Back edges describe descendant-to-ancestor relations, as they lead from “high” to “low” nodes.
 *
 * — Forward Edges        (Only in DIRECTED!)
 *   *GRAY* —> *BLACK*
 *   discovery[a] ≤ discovery[b], pre[v] < pre[w]
 *   Forward edges describe ancestor-to-descendant relations, as they lead from “low” to “high” nodes.
 *
 *
 * — Forward (Left-to-right) Cross Edges  (Only in DIRECTED, but cannot happen!)
 * — Backward (Right-to-left) Cross Edges (Only in DIRECTED!)
 *   pre[v] > pre[w], and post[w] is defined
 *
 *   Cross Edges
 *   *GRAY* —> *BLACK*
 *   discovery[a] > discovery[b]
 *   cross edge if and only if d[v] < f[v] < d[u] < f[u]
 *   They can go between vertices in the same depth-first tree as long as one vertex is not an ancestor of the other, or they can go between vertices in different depth-first trees. Cross edges link nodes with no ancestor-descendant relation and point from “high” to “low” nodes.
 *
 *   The central characteristic of depth-first search in directed graphs is that all cross edges are from higher dfs number to lower dfs numbers.
 */

@protocol EdgeClassifierDelegate <NSObject>

@optional

/*!
 * A tree edge is an edge that is in the output tree of some graph search algorithm.
 */
- (void) edgeClassifiedAsTreeEdge: (edgeType) e;

/*!
 * Back edge: from descendent to ancestor. Note that a self-loop is a back edge.
 */
- (void) edgeClassifiedAsBackEdge: (edgeType) e;

/*!
 * Forward Edge – An edge not in the output tree that connects a vertex to a nonchild descendent in the output tree.
 *
 * Not applicable to undirected graphs!
 */
- (void) edgeClassifiedAsForwardEdge: (edgeType) e;

/*!
 * Cross edges lead to neither descendant nor ancestor; they therefore lead to a node that has already been processed.
 *
 * Never called in a DFSs of undirected graphs!
 */
- (void) edgeClassifiedAsCrossEdge: (edgeType) e;

@end
