////////////////////////////////////////////////////////////////////////////////
//  
//  Graph.h
//  
//  KPMultigraph
//  
//  Created by Konstantin Pavlikhin on 26/08/11.
//  
////////////////////////////////////////////////////////////////////////////////

#import <Foundation/Foundation.h>

#import "ElementTypes.h"

/// Protocol Graph declares method prototypes common to absolutely all of the concrete graph types.
@protocol Graph <NSObject>

/// @name Operations with vertices

#pragma mark - Operations with vertices

/*!
 * Returns a set populated with vertices of the graph.
 *
 * Changes to the graph are not reflected on the returned set. Should return an empty set on null graphs, not just a nil value.
 *
 * @return Possibly empty set of the graph vertices.
 */
- (NSSet*) vertexSet;

/*!
 * Returns a boolean value that indicates whether a given vertex is present in the graph.
 *
 * @param vertex Vertex to test for graph membership. Could be nil.
 *
 * @return YES if vertex is present in the graph, otherwise NO.
 */
- (BOOL) containsVertex: (vertexType) vertex;

/*!
 * Returns the number of vertices currently in the graph.
 *
 * Also called a graph order.
 *
 * @return The number of vertices currently in the graph.
 */
- (NSUInteger) vertexCount;

/*!
 * Returns a degree of the vertex.
 *
 * Also called local degree or valency.
 *
 * @return Vertex degree value. Should return NSNotFound if vertex doesn't belong to the graph.
 */
- (NSUInteger) degreeOfVertex: (vertexType) vertex;

/*!
 * Returns a boolean value that indicates whether a given vertex is odd.
 *
 * A graph vertex is said to be an odd node if its vertex degree is odd.
 *
 * @return YES if vertex is odd, otherwise NO.
 */
- (BOOL) isOddVertex: (vertexType) vertex;

/*!
 * Returns a boolean value that indicates whether a given vertex is even.
 *
 * A graph vertex is said to be an even node if its vertex degree is even.
 *
 * @return YES if vertex is even, otherwise NO.
 */
- (BOOL) isEvenVertex: (vertexType) vertex;

/*!
 * Returns a boolean value that indicates whether a given vertex is isolated.
 *
 * Vertex with 0 degree is called isolated.
 *
 * @return YES if vertex is isolated, otherwise NO.
 */
- (BOOL) isIsolatedVertex: (vertexType) vertex;

/*!
 * Returns a boolean value that indicates whether a given vertex is pendant.
 *
 * Vertex with 1 degree is called pendant.
 *
 * @return YES if vertex is pendant, otherwise NO.
 */
- (BOOL) isPendantVertex: (vertexType) vertex;

/*!
 * Returns an open neighbourhood of a given vertex.
 *
 * The open neighbourhood of a vertex is the set of all the vertices adjacent to it, not including itself.
 *
 * @return Possibly empty set of vertex neighbours or nil if vertex doesn't belong to the graph.
 */
- (NSSet*) openNeighbourhoodSetOf: (vertexType) vertex;

/*!
 * Returns a closed neighbourhood of a given vertex.
 *
 * The closed neighbourhood of a vertex is the set of all the vertices adjacent to it, including itself.
 *
 * @return A set of vertex neighbours, containing at least the vertex itself or nil if vertex doesn't belong to the graph.
 */
- (NSSet*) closedNeighbourhoodSetOf: (vertexType) vertex;

/*!
 * Returns a boolean value that indicates whether given vertices share one incident edge.
 *
 * @return YES if both vertices share the common edge, NO otherwise.
 */
- (BOOL) isVertex: (vertexType) vertex1 neighbourOf: (vertexType) vertex2;

/*!
 * Returns a count of open neighbourhood set of a given vertex.
 *
 * @return The count of vertex neighbours or NSNotFound if given vertex doesn't belong to the graph.
 */
- (NSUInteger) neighboursCountOfVertex: (vertexType) vertex;

/// @name Operations with vertices & edges.

#pragma mark - Operations with vertices & edges

/*!
 * Given edge and one of its endpoints returns the vertex from the opposite side.
 *
 * @param edge The edge to inspect.
 *
 * @param vertex Current vertex.
 *
 * @return The vertex from the opposite side of the edge, nil if vertices aren't adjacent.
 */
- (vertexType) oppositeVertexOfEdge: (edgeType) edge currentVertex: (vertexType) vertex;

/// @name Operations with edges

#pragma mark - Operations with edges

/*!
 * Returns set populated with edges of the current graph.
 *
 * Should return an empty set on edgeless graphs, not just a nil value.
 *
 * @return Possibly empty set of the graph edges.
 */
- (NSSet*) edgeSet;

/*!
 * Returns a boolean value that indicates whether a given edge is present in the graph.
 *
 * @param edge The edge to test for graph membership. Could be nil.
 *
 * @return YES if edge is present in the graph, otherwise NO.
 */
- (BOOL) containsEdge: (edgeType) edge;

/*!
 * Returns the number of edges currently in the graph.
 *
 * @return The number of edges currently in the graph.
 */
- (NSUInteger) edgeCount;

/*!
 * Returns a boolean value that indicates whether a given edge is a loop edge.
 *
 * @param edge The edge to test for being a loop edge.
 *
 * @return YES if edge is a loop edge in the graph, otherwise NO.
 */
- (BOOL) isLoopEdge: (edgeType) edge;

/// @name Universal method for graphs traversal.

#pragma mark - Universal method for graphs traversal

/*!
 * Returns enumerator of edges following which it's possible to "get out" of the given vertex.
 *
 * @return Possibly empty enumerator of outgoing edges, not just a nil for sink-vertices.
 */
- (NSEnumerator*) possibleWaysOutEnumeratorOf: (vertexType) vertex;

/// @name Testing for trivial graph types.

#pragma mark - Testing for trivial graph types

/*!
 * Returns a boolean value that indicates whether a given graph is a null graph.
 *
 * The term "null graph" is used both to refer to any empty graph and to the empty graph on 0 nodes.
 *
 * @return YES if receiver of this message is null graph, NO otherwise.
 */
- (BOOL) isNullGraph;

/*!
 * Returns a boolean value that indicates whether a given graph is a singleton graph.
 *
 * The singleton graph is the graph consisting of a single isolated node with no edges.
 *
 * @return YES if receiver of this message is singleton graph, NO otherwise.
 */
- (BOOL) isSingletonGraph;

/*!
 * Returns a boolean value that indicates whether a given graph consist of vertices only (has 0 edges).
 *
 * @return YES if receiver of this message is an edgeless graph.
 */
- (BOOL) isEdgelessGraph;

@end

#pragma mark - Mutable Version

/// Protocol MutableGraph declares method prototypes common to absolutely all of the concrete mutable graph types.
@protocol MutableGraph <Graph>

/// @name Operations with vertices.

#pragma mark - Operations with vertices

/*!
 * Adds an arbitrary object to the graph.
 *
 * If the object [vertex] already added to the graph this method does nothing.
 *
 * @param vertex Object to be added to the graph. Must not be nil.
 *
 * @exception NSInvalidArgumentException Raises if argument is nil.
 */
- (void) addVertex: (vertexType) vertex;

/*!
 * Removes a given vertex from the graph including all its touching edges if present.
 *
 * If the object [vertex] does not belong to the graph this method does nothing.
 *
 * @param vertex The vertex to remove from graph. Must not be nil.
 *
 * @exception NSInvalidArgumentException Raises if argument is nil.
 */
- (void) removeVertex: (vertexType) vertex;

/*!
 * Removes a given vertices set from the graph.
 *
 * If some verties from the set do not belong to the graph this method ignores them.
 *
 * @param vertices The vertices set to remove from graph. Must not be nil.
 *
 * @exception NSInvalidArgumentException Raises if argument is nil.
 */
- (void) removeVerticesFromSet: (NSSet*) vertices;

/*!
 * Removes all vertices (and all edges) from the graph.
 */
- (void) removeAllVertices;

/// @name Operations with edges.

#pragma mark - Operations with edges

/*!
 * Removes a given edge from the graph.
 *
 * If the object [edge] does not belong to the graph this method does nothing.
 *
 * @param edge The edge to remove from graph. Must not be nil.
 *
 * @exception NSInvalidArgumentException Raise if argument is nil.
 */
- (void) removeEdge: (edgeType) edge;

/*!
 * Removes a given edges set from the graph.
 *
 * If some edges from the set do not belong to the graph this method ignores them.
 *
 * @param edges The edges set to remove from graph. Must not be nil.
 *
 * @exception NSInvalidArgumentException Raise if argument is nil.
 */
- (void) removeEdgesFromSet: (NSSet*) edges;

/*!
 * Removes all edges from the graph, leaving vertices in place.
 */
- (void) removeAllEdges;

@end
