////////////////////////////////////////////////////////////////////////////////
//  
//  GraphTraversal.h
//  
//  KPMultigraph
//  
//  Created by Konstantin Pavlikhin on 27/08/11.
//  
////////////////////////////////////////////////////////////////////////////////

#import "ElementTypes.h"

@protocol Graph, TraversalVisitor;

/*!
 * BFT & DFT algorithms are very similar in many ways, so we need to encapsulate some functionality in this abstract class.
 */
@interface AbstractGraphTraversal : NSObject
{
  @protected
    /// The traversed graph.
    id<Graph> G;
    
    /// Vertex to its mark association.
    NSMapTable* marks;
    
    // Array of traversal visitors.
    NSHashTable* visitors;
}

@property(readwrite, retain) id<Graph> graph;

/*!
 * Convenient way to create traversal algorithm instance.
 */
+ (id) traversal;

/*!
 * Creates NSMapTable suitable to use as vertex to its mark association. Assuming that vertex is an arbitrary object and mark is an immutable NSString.
 *
 * @return Autoreleased NSMapTable.
 */
+ (NSMapTable*) mapTableSuitableForVertexToMarkAssociation;

/*!
 * Subclasses have to define this method with main traversal algorithm behavior. Default implementation just raises an exception.
 *
 * @exception SubclassResponsibility if method is not redefined.
 */
- (void) traverseFromVertex: (vertexType) v;

/*!
 * Marks vertex as *Discovered*.
 */
- (void) discoverVertex: (vertexType) v;

/*!
 * Checks vertex mark and notifies visitors with according event.
 *
 * @return YES if vertex v is undiscovered. NO otherwise.
 *
 * @exception NSInternalInconsistencyException if vertex mark is undefined.
 */
- (BOOL) classifyVertex: (vertexType) v;

typedef void (^undiscoveredProcessor)(vertexType);

/*!
 * Enumerates all outgoing edges of vertex u and performs undiscoveredBlock if head (opposite) vertex was classified as *Undiscovered*
 */
- (void) visitVertex: (vertexType) u undiscoveredProcessor: (undiscoveredProcessor) undiscoveredBlock;

/*!
 * Resets the algorithm to its initial state. Called after -connectedVerticesTraversalDidFinish event to allow you to reuse algorithm instance.
 */
- (void) resetState;

/*!
 * This method performs traversal of the whole graph by performing sequential traversals of a connected components.
 */
- (void) traverseGraph: (id<Graph>) G visitors: (NSObject<TraversalVisitor>*) v1, ... NS_REQUIRES_NIL_TERMINATION;

/*!
 * The visitors are notified in the order as they were passed to the method.
 *
 * @exception NSInvalidArgumentException if some of the arguments is nil.
 */
- (void) traverseGraph: (id<Graph>) G startVertex: (vertexType) s visitors: (NSObject<TraversalVisitor>*) v1, ... NS_REQUIRES_NIL_TERMINATION;

/*!
 * Main template method of abstract connected component traversal algorithm.
 */
- (void) traverseGraph: (id<Graph>) g startVertex: (vertexType) s visitorsList: (va_list) arguments;

@end

/*!
 * The concrete subclass of AbstractGraphTraversal suitable for instantiation.
 */
@interface BreadthFirstTraversal : AbstractGraphTraversal

@end

/*!
 * The concrete subclass of AbstractGraphTraversal suitable for instantiation.
 */
@interface DepthFirstTraversal : AbstractGraphTraversal

@end
