////////////////////////////////////////////////////////////////////////////////
//  
//  GraphTraversal.m
//  
//  KPMultigraph
//  
//  Created by Konstantin Pavlikhin on 27/08/11.
//  
////////////////////////////////////////////////////////////////////////////////

#import "GraphTraversal.h"

#import "TraversalVisitor.h"

#import "Graph.h"

// The vertex is in its initial, virgin state. Also called *White* & *Unexplored*.
//NSString* const undiscovered = @"Undiscovered";

// The vertex has been found, but we have not yet checked out all its incident edges. Also called *Gray* & *Frontier*.
NSString* const discovered = @"Discovered";

// The vertex after we have visited all its incident edges. Also called *Black*.
NSString* const processed = @"Processed";


@implementation AbstractGraphTraversal

@synthesize graph = G;

+ (id) traversal
{
  return [[[[self class] alloc] init] autorelease];
}

+ (NSMapTable*) mapTableSuitableForVertexToMarkAssociation
{
  NSPointerFunctionsOptions keyOptions = NSPointerFunctionsStrongMemory | NSPointerFunctionsObjectPointerPersonality;
  
  NSPointerFunctionsOptions valueOptions = NSPointerFunctionsStrongMemory | NSPointerFunctionsObjectPointerPersonality;
  
  return [NSMapTable mapTableWithKeyOptions: keyOptions valueOptions: valueOptions];
}

- (id) init
{
  self = [super init];
  
  if(!self) return nil;
  
  marks = [[[self class] mapTableSuitableForVertexToMarkAssociation] retain];
  
  // Non-retaining hash table to keep visitors.
  visitors = [[NSHashTable hashTableWithOptions: NSHashTableObjectPointerPersonality] retain];
  
  return self;
}

- (void) dealloc
{
  [G release], G = nil;
  
  [marks release], marks = nil;
  
  [visitors release], visitors = nil;
  
  [super dealloc];
}

- (void) traverseFromVertex: (vertexType) v
{
  [self subclassResponsibility: _cmd];
}

// TODO: Refactor this into something more clean.
#define FOR_EACH_VISITOR for(NSObject<TraversalVisitor>* visitor in visitors)

- (void) discoverVertex: (vertexType) v
{
  [marks setObject: discovered forKey: v];
}

- (BOOL) classifyVertex: (vertexType) v
{
  NSString* Vmark = [marks objectForKey: v];
  
  if(!Vmark)
  {
    FOR_EACH_VISITOR [visitor foundUndiscoveredVertex: v]; return YES;
  }
  
  if([Vmark isEqualTo: discovered])
  {
    FOR_EACH_VISITOR [visitor foundDiscoveredVertex: v]; return NO;
  }
  
  if([Vmark isEqualTo: processed])
  {
    FOR_EACH_VISITOR [visitor foundProcessedVertex: v]; return NO;
  }
  
  [NSException raise: NSInternalInconsistencyException format: nil]; return NO;
}

- (void) visitVertex: (vertexType) u undiscoveredProcessor: (undiscoveredProcessor) undiscoveredBlock
{
  FOR_EACH_VISITOR [visitor willVisitVertex: u];
  
  // Перечисляем все (исходящие) ребра вершины u.
  for(edgeType e in [G possibleWaysOutEnumeratorOf: u])
  {
    FOR_EACH_VISITOR [visitor willTraverseEdge: e];
    
    // У нас есть ребро e и первая из его инцидентных вершин u. Ищем другой конец v...
    vertexType v = [G oppositeVertexOfEdge: e currentVertex: u];
    
    // Если вершина v undiscovered...
    if([self classifyVertex: v])
    {
      [self discoverVertex: v];
      
      undiscoveredBlock(v);
    }
  }
  
  // Пометить вершину u как processed.
  [marks setObject: processed forKey: u];
  
  FOR_EACH_VISITOR [visitor didVisitVertex: u];
}

- (void) resetState
{
  [self setGraph: nil];
  
  [marks removeAllObjects];
  
  [visitors removeAllObjects];
}

- (void) traverseGraph: (id<Graph>) g visitors: (NSObject<TraversalVisitor>*) v1, ...
{
  va_list arguments;
  
  va_start(arguments, v1);
  
  //FOR_EACH_VISITOR [visitor graphTraversalDidStart];
  
  // Обходим граф начиная с произвольной вершины.
  [self traverseGraph: g startVertex: [[g vertexSet] anyObject] visitorsList: arguments];
  
  va_end(arguments);
  
  for(vertexType v in [g vertexSet])
  {
    // Если какая-то вершина осталась ненайденной — производим поиск от нее.
    if(![marks objectForKey: v])
    {
      va_start(arguments, v1);
      
      [self traverseGraph: g startVertex: v visitorsList: arguments];
      
      va_end(arguments);
    }
  }
  
  //FOR_EACH_VISITOR [visitor graphTraversalDidFinish];
  
  [self resetState];
}

- (void) traverseGraph: (id<Graph>) g startVertex: (vertexType) s visitors: (NSObject<TraversalVisitor>*) v1, ...
{
  // A pointer to a list of variable arguments.
  va_list arguments;
  
  // Initializes a va_list to point to the first argument after the argument specified.
  va_start(arguments, v1);
  
  [self traverseGraph: g startVertex: s visitorsList: arguments];
  
  // Releases any memory held by the va_list data structure.
  va_end(arguments);
}

- (void) traverseGraph: (id<Graph>) g startVertex: (vertexType) s visitorsList: (va_list) arguments
{
  if(!g || !s) [NSException raise: NSInvalidArgumentException format: nil];
  
  [self setGraph: g];
  
  //******************************************************************************************************************
  
  NSObject<TraversalVisitor>* v;
  
  [visitors removeAllObjects];
  
  // Fetches the next argument out of the list. You must specify the type of the argument (so that va_arg knows how many bytes to extract).
  while((v = va_arg(arguments, NSObject<TraversalVisitor>*))) [visitors addObject: [v ifResponds]];
  
  //******************************************************************************************************************
  
  // По идее, из множества вершин графа надо было выкинуть стартовую вершину...
  for(vertexType u in [G vertexSet])
  {
    FOR_EACH_VISITOR [visitor initializeVertex: u];
    
    // Пометить u как undiscovered.
    //[marks setObject: undiscovered forKey: u];
  }
  
  FOR_EACH_VISITOR [visitor willStartFromVertex: s];
  
  FOR_EACH_VISITOR [visitor connectedVerticesTraversalDidStart];
  
  // Пометить стартовую вершину s как discovered.
  [self classifyVertex: s];
  
  [self discoverVertex: s];
  
  [self traverseFromVertex: s];
  
  FOR_EACH_VISITOR [visitor connectedVerticesTraversalDidFinish];
}

@end

#pragma mark - BreadthFirstTraversal

@implementation BreadthFirstTraversal

- (void) traverseFromVertex: (vertexType) s
{
  // Добавить стартовую вершину s в очередь Q.
  NSMutableArray* Q = [NSMutableArray arrayWithObject: s];
  
  // Пока очередь Q не закончится, вынимать вершины u из очереди.
  vertexType u = nil;
  
  while((u = [Q dequeue]))
  {
    /// For each *Undiscovered* vertex, adjacent to u — add it to the processing queue.
    [self visitVertex: u undiscoveredProcessor: ^(vertexType v){ [Q enqueue: v]; }];
  }
}

@end

#pragma mark - DepthFirstTraversal

@implementation DepthFirstTraversal

- (void) traverseFromVertex: (vertexType) u
{
  /// For each *Undiscovered* vertex, adjacent to u — dive in its first descendant.
  [self visitVertex: u undiscoveredProcessor: ^(vertexType v){ [self traverseFromVertex: v]; }];
}

@end
