////////////////////////////////////////////////////////////////////////////////
//  
//  UndirectedGraph.h
//  
//  KPMultigraph
//  
//  Created by Konstantin Pavlikhin on 30/08/11.
//  
////////////////////////////////////////////////////////////////////////////////

#import "Graph.h"

/// Protocol UndirectedGraph declares method prototypes common to absolutely all of the concrete undirected graph subtypes.
@protocol UndirectedGraph <Graph>

/// @name Operations with vertices & edges.

#pragma mark - Operations with vertices & edges

/*!
 * Returns an undirected edge between given vertices.
 *
 * @return An edge connecting given vertices, if these vertices exist in graph. nil otherwise.
 */
- (edgeType) edgeBetween: (vertexType) u and: (vertexType) v;

/*!
 * Returns all undirected edges between given vertices.
 *
 * @return A set of edges that connect given vertices, if these vertices exist in graph. Empty set otherwise.
 */
- (NSSet*) allEdgesBetween: (vertexType) u and: (vertexType) v;

/*!
 * Tests graph for containing an edge between given vertices.
 *
 * @return YES if there is an edge between u & v. NO otherwise.
 */
- (BOOL) containsEdgeBetween: (vertexType) u and: (vertexType) v;

/*!
 * Returns a Pair of vertices incident to the given edge.
 *
 * @return Pair of vertices or empty pair, not just nil.
 */
- (Pair*) incidentVerticesPairOf: (edgeType) edge;

/*!
 * Returns a Set of edges incident to the given vertex.
 *
 * @return Possibly empty Set of edges incident to the given vertex.
 */
- (NSSet*) incidentEdgesSetOf: (vertexType) vertex;

/*!
 * Tests a given vertex and edge for being incident to each other.
 *
 * @return YES if given vertex and edge are mutably incident to each other, NO otherwise.
 */
- (BOOL) isVertex: (vertexType) vertex incidentToEdge: (edgeType) edge;

/*!
 * Tests a given edge and vertex for being incident to each other.
 *
 * @return YES if given edge and vertex are mutably incident to each other, NO otherwise.
 */
- (BOOL) isEdge: (edgeType) edge incidentToVertex: (vertexType) vertex;

#pragma mark - Operations with edges

/*!
 * @return Possibly empty Set of edges adjacent to the given edge.
 */
- (NSSet*) adjacentEdgesSetOf: (edgeType) edge;

/*!
 * @return YES if two given edges are adjacent to each other.
 */
- (BOOL) isEdge: (edgeType) edge1 adjacentTo: (edgeType) edge2;

@end

#pragma mark - Mutable Version

@protocol MutableUndirectedGraph <MutableGraph, UndirectedGraph>

/*!
 * Adds specified edge between given vertices.
 *
 * @param edge Edge object to be added to the graph. Must not be nil.
 *
 * @param u Some vertex of the graph. Must not be nil.
 *
 * @param v Some vertex of the graph. Must not be nil.
 *
 * @exception NSInvalidArgumentException if some argument is nil.
 *
 * @see addEdge:betweenVerticesPair:
 */
- (void) addEdge: (edgeType) edge between: (vertexType) u and: (vertexType) v;

/*!
 * @see addEdge:between:and:
 */
- (void) addEdge: (edgeType) edge betweenVerticesPair: (Pair*) pair;

/*!
 * Removes an edge between vertex u and v.
 *
 * @return The removed edge or nil if no edge was removed.
 */
- (edgeType) removeEdgeBetween: (vertexType) u and: (vertexType) v;

@end
